%-------------------------------------------------------------------------%
% Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
%                               http://laboratoirehubertcurien.fr/
% 
% Contact : corinne.fournier@univ_st_etienne.fr
% May 2012
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%
%
% Goal : Simple introductory example of the usage of the toolbox using
% multi-scale approach
%-------------------------------------------------------------------------%
%% Choose the data you are planning to process
DATAH=1; 
% For DATAH==1,2 there is no need to change the parameters
% DATAH :
% 1 if simulations
% 2 if holograms of water droplets from LMFA
% 3 for provided holograms - NOTE!!!!! please make sure to set all the parameters accordingly


%% Check the corresponding cell to set the parameters


%%
if DATAH==1
    %% Simulation parameters - LMFA water droplets

   
    lambda=0.532e-6; % wavelenght of the laser beam in meter

    pix=21.7e-6; % pixel size of the camera in meter
    
    nn=8; % number of particles

    hol_size=[1024 1280]; % sensor size in pixels
    
    Num_holo=4;% number of holograms in the video
    
    magni=1.42; % magnification factor

     % ranges = [Zmin Zmax;Rmin Rmax] in meter for object space(real values without
     % magnification)
    ranges=[0.39-0.09 0.39+0.09;27.5e-6 32.5e-6].*magni;
   
    mrate=[2 2]; % rate of dimensions of the area of interest over the sensor's dimensions

    Mask=0; % Mask with the same size as the holograms to exclude pixels from the calculations - (0 if there is no Mask)

    Alpha_thresh=0.5; % (recommended-enough : 0) - threshold on maximum of likelihood maps - Alpha is basically equal to 2*Intensity of laser beam. 
    % In practice alpha changes from 2 to several thousands. In any case a threshold of 0 will always find particles and
    % not ghost particles, but if alpha is a factor of 1000, a small threshold on alpha can cause several detection of one
    % particle which is caused by the residuals on the hologram. These
    % residuals are the results of the non-perfect formulation of the model  

    K=sqrt(0.7); % fill-factor of the camera

    itter=15; % (recommended-enough : 15 for in the field, 30 for out of field) maximum number of iterations of optimization - 

    coef=100;% (recommended-enough : 10) coef*CRLB will be used as a lower limit on the accuracy of optimization

    SecondT=0; % 0 for real data, 1 for simulated holograms 

    UNIF_Z=1; % (recommended-faster : 1) 1 if uniform sampling of Z is enough, 0 otherwise 

    q=10; % (recommended-enough : 10) the number of preserved fringes on the down-sampled holograms
     
    
    NoFigure=1;% 1 if you don't want to see the hologram after each particle cleaned from the hologram
    MEXQmap=0; % 0 (recommended-faster : 0)
    MEXmodelFAST=1; % (recommended-faster : 1) 1 if fast implementation of model with multi-threading is required 


%     holsize=hol_size.*mrate;

%     coordinates=zeros(Num_holo*nn,4);
%     coordinates(:,1)=rand(Num_holo*nn,1).*holsize(1)-holsize(1)/2-1;
%     coordinates(:,2)=rand(Num_holo*nn,1).*holsize(2)-holsize(2)/2-1;
%     coordinates(:,3)=rand(Num_holo*nn,1).*(ranges(1,2) -ranges(1,1)) +ranges(1,1);
%     coordinates(:,4)=rand(Num_holo*nn,1).*(ranges(2,2) -ranges(2,1)) +ranges(2,1);
%     save coordinates_simul.mat coordinates


    load coordinates_simul.mat

    sigma=0.5;
    A0=100;
    SNR=A0/sigma;
    REMOVED_I0=A0;
%     
%     for i=1:Num_holo
%         FOV=1;
%         i
%         [hol,particles,REMOVED_I0,noise]=simulate_holo_mex(nn,lambda,pix,holsize(1),holsize(2),A0,SNR,ranges(1,:),ranges(2,:),0,coordinates((i-1)*nn+1:i*nn,:),0,K);
%         holo_simul(:,:,i)=hol(floor(holsize(1)/2)-floor(hol_size(1)/2)+1:floor(holsize(1)/2)+floor(hol_size(1)/2),floor(holsize(2)/2)-floor(hol_size(2)/2)+1:floor(holsize(2)/2)+floor(hol_size(2)/2));
%   
% %          figure;imshow(holo_simul(:,:,i),[]);   
%     end
%     save holograms_simul.mat holo_simul
    
    load holograms_simul.mat
    holograms=holo_simul;
    clear holo_simul        
%% experimental parameters - holograms of water droplets from LMFA
elseif DATAH==2

    lambda=0.532e-6; % wavelenght of the laser beam in meter

    pix=21.7e-6; % pixel size of the camera in meter

    load holo_water_LMFA.mat % holograms with their removed mean hologram
    holograms=holo_water;
    clear holo_water

    load I_water_LMFA.mat % removed mean of the holograms which is a hologram itself
    REMOVED_I0=mean(I(:));
    clear I
    
    hol_size=size(holograms(:,:,1)); % sensor size in pixels
    Num_holo=size(holograms,3);% number of holograms in the video
    
    magni=1.42; % magnification factor

     % ranges = [Zmin Zmax;Rmin Rmax] in meter for object space(real values without
     % magnification)
    ranges=[0.39-0.09 0.39+0.09;27.5e-6 32.5e-6].*magni;
   
    mrate=[2 2]; % rate of dimensions of the area of interest over the sensor's dimensions

    Mask=0; % Mask with the same size as the holograms to exclude pixels from the calculations - (0 if there is no Mask)

    Alpha_thresh=0.5; % (recommended-enough : 0) - threshold on maximum of likelihood maps - Alpha is basically equal to 2*Intensity of laser beam. 
    % In practice alpha changes from 2 to several thousands. In any case a threshold of 0 will always find particles and
    % not ghost particles, but if alpha is a factor of 1000, a small threshold on alpha can cause several detection of one
    % particle which is caused by the residuals on the hologram. These
    % residuals are the results of the non-perfect formulation of the model  

    K=sqrt(0.7); % fill-factor of the camera

    itter=15; % (recommended-enough : 15 for in the field, 30 for out of field) maximum number of iterations of optimization - 

    coef=100;% (recommended-enough : 10) coef*CRLB will be used as a lower limit on the accuracy of optimization

    SecondT=0; % 0 for real data, 1 for simulated holograms 

    UNIF_Z=1; % (recommended-faster : 1) 1 if uniform sampling of Z is enough, 0 otherwise 
    
    q=10; % (recommended-enough : 10) the number of preserved fringes on the down-sampled holograms
    
    NoFigure=1;% 1 if you don't want to see the hologram after each particle cleaned from the hologram
    MEXQmap=0; % 0 (0 recommended)
    MEXmodelFAST=1; % 1 if fastimplementation of model with multi-threading is required (1 recommended)

    % SNR and sigma of noise 
    II0=holograms(:,:,1)-mean(mean(holograms(:,:,1)));

    % NOTE: after removing the background from the holograms, you need to
    % specify the coordinates of one window on the background which contains 
    % only noise and one window on the patterns of particles for signal.
    % The matrix should be provided as [X_top_left X_bottom_right;Y_top_left Y_bottom_right]

    % Note!! Data Cursor on figure of Matlab provides the coordinates of ever pixel as (y,x). 
    % The windows should contain more than 50 pixels.

    % the foloowing values are set for the holograms of water droplets from LMFA
    Window_noise=[70 200;530 630];% 
    Window_signal=[443 466; 1025 1037];% 

    sigma=std(std(II0(Window_noise(1,1):Window_noise(1,2),Window_noise(2,1):Window_noise(2,2))));
    A0=mean(mean(II0(Window_signal(1,1):Window_signal(1,2),Window_signal(2,1):Window_signal(2,2))));

    

    %% experimental parameters - unknown holograms 
    % please make sure to set all the parameters accordingly
elseif DATAH==3

    lambda=[]; % wavelenght of the laser beam in meter

    pix=[]; % pixel size of the camera in meter

    load UN_holo.mat; % holograms with their removed mean hologram -  
%     this matrix should be of dimensions : NX*NY*Num_holo where NX is the number of pixels on vertical side, 
%       NY is the number of pixels on horizontal side and Num_holo is the number of holograms in the video. 

    holograms=[]; % give the name of the matrix which containes the holograms
    % for example: a matrix called holo is saved in the file
    % UN_holo.mat => holograms=holo;
    
    load UN_Background.mat % removed mean of the holograms which is a hologram itself - 
    % this matrix should be of dimensions : NX*NY*1
    
    I=[];% give the name of the matrix which containes the background
    % for example: a matrix called BG is saved in the file
    % UN_Background.mat => I=BG;
    
    REMOVED_I0=mean(I(:));
    clear I
    hol_size=size(holograms(:,:,1)); % sensor size in pixels
    Num_holo=size(holograms,3);% number of holograms in the video
   
    magni=[]; % magnification factor

     % ranges = [Zmin Zmax;Rmin Rmax] in meter for object space(real values without
     % magnification)
    ranges=[].*magni;
   
    mrate=[]; % rate of dimensions of the area of interest over the sensor's dimensions

    Mask=0; % Mask with the same size as the holograms to exclude pixels from the calculations - (0 if there is no Mask)

    Alpha_thresh=0.5; % (recommended-enough : 0) - threshold on maximum of likelihood maps - Alpha is basically equal to 2*Intensity of laser beam. 
    % In practice alpha changes from 2 to several thousands. In any case a threshold of 0 will always find particles and
    % not ghost particles, but if alpha is a factor of 1000, a small threshold on alpha can cause several detection of one
    % particle which is caused by the residuals on the hologram. These
    % residuals are the results of the non-perfect formulation of the model  

    K=[]; % fill-factor of the camera

    itter=15; % (recommended-enough : 15 for in the field, 30 for out of field) maximum number of iterations of optimization - 

    coef=100;% (recommended-enough : 10) coef*CRLB will be used as a lower limit on the accuracy of optimization

    SecondT=0; % 0 for real data, 1 for simulated holograms 

    UNIF_Z=1; % (recommended-faster : 1) 1 if uniform sampling of Z is enough, 0 otherwise 
    
    q=10; % (recommended-enough : 10) the number of preserved fringes on the down-sampled holograms

    NoFigure=1;% 1 if you don't want to see the hologram after each particle cleaned from the hologram
    MEXQmap=0; % 0 (0 recommended)
    MEXmodelFAST=1; % 1 if fastimplementation of model with multi-threading is required (1 recommended)

    % SNR and sigma of noise 
    II0=holograms(:,:,1)-mean(mean(holograms(:,:,1)));

    % NOTE: after removing the background from the holograms, you need to
    % specify the coordinates of one window on the background which contains 
    % only noise and one window on the patterns of particles for signal.
    % The matrix should be provided as [X_top_left X_bottom_right;Y_top_left Y_bottom_right]

    % Note!! Data Cursor on figure of Matlab provides the coordinates of ever pixel as (y,x). 
    % The windows should contain more than 50 pixels.

    % the foloowing values are set for the holograms of water droplets from LMFA
    Window_noise=[];% 
    Window_signal=[];% 

    sigma=std(std(II0(Window_noise(1,1):Window_noise(1,2),Window_noise(2,1):Window_noise(2,2))));
    A0=mean(mean(II0(Window_signal(1,1):Window_signal(1,2),Window_signal(2,1):Window_signal(2,2))));

end
%%
SNR=A0/sigma;
precise_MR=zeros(20,8,Num_holo);
t1=zeros(20,2,Num_holo);

%% parameters of search space sampling required for matching pursuit step based on the size of basin
% you can change the coefficients of dz and dr to control the sample size
    [deltax,deltay,dz,dr]=CRLB_point(0,0,ranges(1,1)+(ranges(1,2)-ranges(1,1))/2,ranges(2,1)+(ranges(2,2)-ranges(2,1))/2,pix,lambda,hol_size,mrate,sigma,K,0);
    factors=[10*dz 5*dr]; % [Z_step R_step] sampling step size for Z and R in meter in mathcing pursuit 
    factors_FAST=[20*dz 10*dr coef];% [Z_step R_step coef] sampling step size for Z and R in meter in mathcing pursuit
    % coef will be multiplied by the CRLB to stop optimization
%%
% precise_NORM=zeros(20,8,Num_holo);
% t2=zeros(20,2,Num_holo);

for i=1:Num_holo
    FOV=1;
    fprintf('***************************************************************************\n')
    fprintf('***************************************************************************\n')
    fprintf('Processing of the hologram number %g out of %g\n \n',i,Num_holo)

    fprintf('- FAST is started\n')

    [hol1,found,precise,t]=FAST(holograms(:,:,i),mrate,pix,lambda,ranges,FOV,REMOVED_I0,Mask,Alpha_thresh,factors_FAST,K,q,itter,SecondT,UNIF_Z,A0,sigma,MEXQmap,MEXmodelFAST,NoFigure);
%     t1(i,1:size(t,2))=t;
        t1(1:size(t,1),:,i)=t;
    
%         found_MR(1:size(found,1),:,i)=found;
    precise_MR(1:size(precise,1),:,i) =precise;

clear precise hol1 found t
%         FOV=1;
%         [hol2,found,precise,t]=Inverse_approach(holograms(:,:,i),mrate,pix,lambda,ranges,FOV,REMOVED_I0,sigma,Mask,Alpha_thresh,K,itter,10,factors,SecondT,UNIF_Z,MEXQmap,MEXmodelFAST,NoFigure);
%     t2(1:size(t,1),:,i)=t;
%     
% %          found_NORM(1:size(found,1),:,i)=found;
%     precise_NORM(1:size(precise,1),:,i) =precise;  
%     
% clear precise hol2 found t

end

%% Save the foudn coordinates in .mat files. select the names of the out-put files
save(strcat(path_data,'t_toolbox_FAST.mat'),'t1');
% save t_toolbox_FAST.mat t1

save(strcat(path_data,'precise_toolbox_FAST_MR'),'precise_MR');
% save precise_toolbox_FAST_MR.mat precise_MR


% save found_toolbox_FAST_NORM.mat precise_NORM
