function [found_x,found_y,found,p_max,m]=global_detection(hol,w,lambda,pix,ind,temp,ranget,sizex,w_prim,MapFOVx,MapFOVy,tt,N,factors,UNIF_Z,MEXQmap,MEXmodelFAST)
%-------------------------------------------------------------------------%
% Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
%                               http://laboratoirehubertcurien.fr/
% 
% Contact : corinne.fournier@univ_st_etienne.fr
% Sept 2011
%-------------------------------------------------------------------------%
% Goal : Brute-Force search (global detection) in 4D search space for
%       spherical particles : sub-sampling of Z and R parameters
%       Non-uniform sampling technique
%-------------------------------------------------------------------------%
%
%
% inputs:
% hol                   = hologram
% w                     = wighting matrix or Mask
% lambda                = Wavelength in meter
% pix                   = pixel size in meter
% ind                   = 1 to search for z, 2 to search for r
% temp                  = regarding the value of ind, if ind=1 temp
%                           contains the r of the particle; if ind=2 temp 
%                           contains the z of the particle
% ranget                = range of the variable under search a 1-by-2 matrix
% sizex                 = width of the original hologram
% w_prim                = wigthing matrix for Out-of-Field detection
% 
% MapFOVx,MapFOVy       = indices of area on map, where one would like to find particles ,
%                          [x_first:x_last] if arg #15 MEXQmap=0 ; 
%                          [x_first x_last] if MEXQmap=1 ;
%                       i.g. if field of interest is twice of the size of hologram, 
%                       then the map is 3 times of the size of hologram ,and 
%                       map_FOV1,map_FOV2 show an area with a size twice of the hologram size
%                       in the middle of the map.
% tt                    = (downsampling) ? (downsampling rate in pixels : fill-factor) 
% N                     = (downsampling) ? (downsampling rate in pixels : 1) 
% factors               = [Z_step R_step]; sampling step size in matching pursuit step
% UNIF_Z                = 1 if unifrom sampling of Z is required
% MEXQmap                = 1 if mex function of Qmap should be used
% MEXmodelFAST           = 1 if model with multi-threading is required
%-------------------------------------------------------------------------%
%
% outputs:
% found_x               = X coordinate of the found particle centered on the center of image in pixel
% found_y               = Y coordinate of the found particle centered on the center of image in pixel
% found                 = regarding the value of ind, found contains the found z or r coordinate of the particle in meter
% p_max                 = value of the peak of Qmap for the found particle
% m                     = number of calculated Qmaps
%-------------------------------------------------------------------------%
%
% To test:
% n=5; lambda=0.532e-6; pix=7e-6; holsize=[1024 1024]; I00= 5; SNR=100;
% ranget=[0.2 0.25;50e-6 55e-6]; 
% coordinates=[0 0 0.2 51e-6;-120 -200 0.23 52e-6;200 120 0.24 50e-6;-200 300 0.245 53.5e-6;100 -500 0.241 52.4e-6];
% meanh=0; ind=1;  K=sqrt(0.7);
% [holo,particle,Mmean,noise]=simulate_holo_mex(n,lambda,pix,holsize(1),holsize(2),I00,SNR,ranget(1,:),ranget(2,:),0,coordinates,meanh,K);
% hol=zeros(2*holsize);
% hol(holsize(1)/2+1:3*holsize(1)/2,holsize(2)/2+1:3*holsize(2)/2)=holo;
% w=zeros(2*holsize);w(holsize(1)/2+1:3*holsize(1)/2,holsize(2)/2+1:3*holsize(2)/2)=1;
% MapFOVx=[floor(holsize(1)/2)+1:3*floor(holsize(1)/2)];MapFOVy=[floor(holsize(2)/2)+1:3*floor(holsize(2)/2)];
% factors=[0.002 1e-6]; ranget=ranget(1,:); sizex=holsize(1);
% w_prim=ones(2*holsize);tt=K;N=0;UNIF_Z=1;
% temp=52e-6;MEXQmap=0;MEXmodelFAST=0;
% [found_x,found_y,found,p_max,m]=global_detection(hol,w,lambda,pix,ind,temp,ranget,sizex,w_prim,MapFOVx,MapFOVy,tt,N,factors,UNIF_Z,MEXQmap,MEXmodelFAST);
% -------------------------------------------------------------------------%
%!-------------------------------------------------------------------------%
%! processing time : ~48.601 s on Cetautomatix .  
%! profiler : global_detection=0.050s, Qmap_mex=48.551s(24 calls) 
%-------------------------------------------------------------------------%
% functions used : Qmap_mex.cpp , max_Qmap.m
%-------------------------------------------------------------------------%
% [1] F. Soulez, L. Denis, E. Thiebaut, C. Fournier, et C. Goepfert, "Inverse problem approach in particle digital holography:
%  out-of-field particle detection made possible", Journal of the Optical Society of America A, vol. 24, no. 12, p. 3708-3716, 2007
%-------------------------------------------------------------------------%


    if ind==1 % we know r which is in temp
        if (UNIF_Z~=1)% Non-niform sampling of Z
            i=1;
            z_ini=ranget(1);
            d_z=factors(1)*4*lambda*((z_ini).^2)/((sizex*N*pix).^2)/2;

            sample_z(i)=z_ini;
            i=i+1;
            sample_z(i)=z_ini+d_z/2;
            z_new=z_ini+d_z; 



            while z_new< ranget(2)

                d_z=factors(1)*4*lambda*((z_new).^2)/((sizex*N*pix).^2)/2;

                if z_new+d_z/2>ranget(2)
                    break;
                end
                i=i+1;
                sample_z(i)=z_new+d_z/2;


                if z_new+d_z>ranget(2)
                    break;
                end

                z_new=z_new+d_z;

            end

            m=size(sample_z,2);
            
            if numel(sample_z)==0
                sample_z(1)=ranget(1)+(ranget(2)-ranget(1))/2;
            end
            fprintf(' - Number of Z samples is %g\n',m)
            row_p=zeros(size(sample_z));
            col_p=zeros(size(sample_z));
            maxim=zeros(size(sample_z));
    %         matlabpool open 6
            for p=1:size(sample_z,2)
                if (MEXQmap==1)
                   [row_p(1,p),col_p(1,p),maxim(1,p),q]=Qmap_mex(hol,lambda,pix,w,temp,sample_z(p),w_prim,MapFOVx,MapFOVy,tt); 
                else
                    [row_p(1,p),col_p(1,p),maxim(1,p),q]=max_Qmap(hol,lambda,pix,w,temp,sample_z(p),w_prim,MapFOVx,MapFOVy,tt,MEXmodelFAST);
                                                                    
                end
                 clear q
            end
%          matlabpool close  
        


        else % Uniform sampling of Z
            delta_z=factors(1);
            
            m=floor((ranget(2)-ranget(1))./delta_z);
            
            if (m==0)
                m=1;
                delta_z=ranget(1)+(ranget(2)-ranget(1))/2;
            end
            fprintf('- Number of Z samples is %g\n',m)
            sample_z=zeros(1,m);
            row_p=zeros(size(sample_z));
            col_p=zeros(size(sample_z));
            maxim=zeros(size(sample_z));
        
            for p=1:m
                sample_z(p)=ranget(1)+(p-1)*delta_z;
                if (MEXQmap==1)
                 [row_p(1,p),col_p(1,p),maxim(1,p),q]=Qmap_mex(hol,lambda,pix,w,temp,sample_z(p),w_prim,MapFOVx,MapFOVy,tt);
                else
                 [row_p(1,p),col_p(1,p),maxim(1,p),q]=max_Qmap(hol,lambda,pix,w,temp,sample_z(p),w_prim,MapFOVx,MapFOVy,tt,MEXmodelFAST);
                end
                 clear q
            end
        end
        
            [p_maxi]=find(maxim==max(max(maxim)));
            p_max=p_maxi(floor(size(p_maxi,1)/2)+1);
            found_x=row_p(p_max);
            found_y=col_p(p_max);
            found_z=sample_z(p_max);
            
            if (numel(sample_z)>2)
                if p_max==1
                    y=[found_z,sample_z(2),sample_z(3)];
                    out_y=[maxim(p_max),maxim(2),maxim(3)];
                elseif p_max==size(sample_z,2)
                    y=[sample_z(p_max-2),sample_z(p_max-1),found_z];
                    out_y=[maxim(p_max-2),maxim(p_max-1),maxim(p_max)];
                else
                    y=[sample_z(p_max-1),found_z,sample_z(p_max+1)];
                    out_y=[maxim(p_max-1),maxim(p_max),maxim(p_max+1)];
                end

                p_y = polyfit(y,out_y,2);
                if p_y(1)==0
                    found=found_z;
                else
                    found=-p_y(2)/(2*p_y(1));
                    if ((found<min(y)) || (found>max(y))) 
                        found=found_z;
                    end

                end
            else
                found=found_z;
            end
         
        
    elseif ind==2 % we know z which is in temp
        
         r=ranget(1);
         range=ranget(2)-ranget(1); % range of variation of z

         delta_r=factors(2);
        
        
         m=floor(range/delta_r);
         if m==0
             m=1;
             delta_r=ranget(1)+(ranget(2)-ranget(1))/2;
         end
         fprintf('- Number of R samples is %g\n',m)
        if m>1
                row_p=zeros(1,m);
                col_p=zeros(1,m);
                maxim=zeros(1,m);

        %         matlabpool open 6
               for p=1:m
                   if (MEXQmap==1)
                       [row_p(p),col_p(p),maxim(p),q]=Qmap_mex(hol,lambda,pix,w,r+p*delta_r,temp,w_prim,MapFOVx,MapFOVy,tt);   
                   else
                        [row_p(1,p),col_p(1,p),maxim(1,p),q]=max_Qmap(hol,lambda,pix,w,r+p*delta_r,temp,w_prim,MapFOVx,MapFOVy,tt,MEXmodelFAST);
                   end
                    clear q
                end
        %          matlabpool close

                [p_maxi]=find(maxim==max(max(maxim)));
                p_max=p_maxi(1);
                found_x=row_p(p_max);
                found_y=col_p(p_max);
        %         [p_max]=find(row_p==found_x && col_p==found_y);
                if m>2
                        found_r=r+p_max*delta_r;
                        if p_max==1
                            y=[found_r,r+(p_max+1)*delta_r,r+(p_max+2)*delta_r];
                            out_y=[maxim(p_max),maxim(p_max+1),maxim(p_max+2)];
                        elseif p_max==m
                            y=[r+(p_max-2)*delta_r,r+(p_max-1)*delta_r,found_r];
                            out_y=[maxim(p_max-2),maxim(p_max-1),maxim(p_max)];
                        else
                            y=[r+(p_max-1)*delta_r,found_r,r+(p_max+1)*delta_r];
                            out_y=[maxim(p_max-1),maxim(p_max),maxim(p_max+1)];
                        end

                        p_y = polyfit(y,out_y,2);

                        if p_y(1)==0
                            found=found_r;
                        else
                            found=-p_y(2)/(2*p_y(1));
                            if found<min(y) || found>max(y) 
                                found=found_r;
                            end
                        end
                 else
                        found_x=row_p(p_max);
                        found_y=col_p(p_max);
                        found=r+p_max*delta_r;
                end
        else
                        found_x=0;
                        found_y=0;
                        found=ranget(1)+(ranget(2)-ranget(1))/2;
                        p_max=1;
        end
            

    end
% end
