function [row,col,maxim,q0]=max_Qmap(hol,lambda,pix,w,r,z,w_prim,map_FOV1,map_FOV2,tt,MEXmodelFAST)
%-------------------------------------------------------------------------%
% Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
%                               http://laboratoirehubertcurien.fr/
% 
% Contact : corinne.fournier@univ_st_etienne.fr
% Sept 2011
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%
% Goal : calculation of Qmap and searching for the pick of Qmap
%
%-------------------------------------------------------------------------%--%
% inputs     
% hol                  = Hologram
% lambda               = wavelenght in meter
% pix                  = pixel size in meter
% w                    = wieghting matrix
% r                    = Radius of Particle in meter
% z                    = axial distance between particle and sensor in meter
% w_prim               = wigthing matrix for Out-of-Field detection
% map_FOV1,map_FOV2    = indices of area on map, where one would like to find particles i.g. if field of interest is twice of the size of hologram, then the map is 3 times of the size of hologram ,and map_FOV1,map_FOV2 show an area with a size twice of the hologram size in the middle of the map
% tt                   = (downsampling) ? (downsampling rate in pixels : fill-factor)
% MEXmodelFAST         = 1 if model with multi-threading is required
%-------------------------------------------------------------------------%
% outputs        
% row,col:      absolute indexes of the maximum of Qmap in pixel, not centered 
% maxim:        the value of maximum of Qmap
% q0 :          Qmap
%-------------------------------------------------------------------------%
%
% To test:
%  n=5; lambda=0.532e-6; pix=7e-6; holsize=[1024 1024]; I00= 5;
% SNR=1000; FOV=1;k=sqrt(0.7);
% ranget=[0.2 0.4;20e-6 55e-6]; 
% coordinates=[0 0 0.2 21e-6;-120 -200 0.28 29e-6;200 120 0.35 40e-6;-200 300 0.39 35e-6;100 -500 0.3 30e-6];
% meanh=0; ind=1; 
% [hol,particle,Mmean,noise]=simulate_holo_mex(n,lambda,pix,holsize(1),holsize(2),I00,SNR,ranget(1,:),ranget(2,:),0,coordinates,meanh,k);
% holo=zeros(2*holsize);
% holo(holsize(1)/2+1:3*holsize(1)/2,holsize(2)/2+1:3*holsize(2)/2)=hol;
% w=zeros(2*holsize);w(holsize(1)/2+1:3*holsize(1)/2,holsize(2)/2+1:3*holsize(2)/2)=1;
% w_prim=ones(2*holsize);
% map_FOV1=holsize(1)/2+1:3*holsize(1)/2;map_FOV2=holsize(2)/2+1:3*holsize(2)/2;
% [row,col,maxim,q0]=max_Qmap(holo,lambda,pix,w,29e-6,0.28,w_prim,map_FOV1,map_FOV2,k,1);
% 
%-------------------------------------------------------------------------%
% [1] F. Soulez, L. Denis, E. Thiebaut, C. Fournier, et C. Goepfert, "Inverse problem approach in particle digital holography:
%  out-of-field particle detection made possible", Journal of the Optical Society of America A, vol. 24, no. 12, p. 3708-3716, 2007
%-------------------------------------------------------------------------%


if MEXmodelFAST==1
      gzero=model_calc_faster(0,0,z,r,pix,lambda,size(hol),0,tt);
      
else
         gzero=model_mex(0,0,z,r,pix,lambda,size(hol),0,tt);
end

    
    w_fft=fft2(w);
    g_fft=fft2(gzero);
      
    wg=ifftshift(ifft2(w_fft.*g_fft)); % wg
    wg=wg(map_FOV1,map_FOV2).^2;

    
    wrg=ifftshift(ifft2(fft2(w.*hol).*g_fft)); %wrg
    wrg=wrg(map_FOV1,map_FOV2);

    
    wg2=ifftshift(ifft2(w_fft.*fft2(gzero.^2)));%wg2
    wg2=wg2(map_FOV1,map_FOV2);
    
    ws=sum(sum(w));

    alpha=-ws.*wrg./(ws.*wg2 - wg);
    
    alpha(find(alpha>0))=1;
    alpha(find(alpha<0))=0;
    

   q0(1:size(map_FOV1,2),1:size(map_FOV2,2))=alpha.*(ws.*(wrg.^2)./(ws.*wg2 - wg)).*w_prim(map_FOV1,map_FOV2);

% if w_prim(floor(end/2),floor(end/2))==0
%     figure;imshow(q0,[]);title('Qmap');colorbar
% end

% find the maximum of the Qmap
    [rowi,coli]=find(q0==max(max(q0)));
    row=rowi(floor(end/2)+1);
    col=coli(floor(end/2)+1);
    maxim=max(max(q0));
   

end